import java.util.List;

import models.Book;
import models.Category;
import models.User;

import org.junit.Before;
import org.junit.Test;

import play.test.Fixtures;
import play.test.UnitTest;

public class BasicTest extends UnitTest {

	@Before
	public void setup() {
		Fixtures.deleteDatabase();
	}

	@Test
	public void createAndRetrieveCategory() {

		// Create a new Category and save it
		new Category("Mineralogie",
				"Mineralien sind das schoenste auf der Welt.").save();
		// Retrieve Category by title and check if not null
		Category catMinerals = Category.find("byTitle", "Mineralogie").first();
		assertNotNull(catMinerals);

	}

	@Test
	public void createBook() {

		// Create a new Category and save it
		Category category = new Category("Römer",
				"Die Römer waren die Besten der Welt.").save();

		// Create a new Book and save it

		new Book("Marc Aurel", "Marc Aurel war sehr ein wichtiger Römer.",
				"Buchrücken angerissen", "Gustav Gans", 250, 1, category)
				.save();

		// Test that the Book has been created

		assertEquals(1, Book.count());

		// Retrieve all Books with Category "Römer"

		List<Book> roemerBooks = Book.find("byCategory", category).fetch();

		// Tests
		assertEquals(1, roemerBooks.size());
		Book firstBook = roemerBooks.get(0);
		assertNotNull(firstBook);
		assertEquals(category, firstBook.category);
		assertEquals("Marc Aurel", firstBook.title);
		assertNotNull(firstBook.insertedAt);

	}

	@Test
	public void addBooksToCategory() {

		// Create a new Category and save it
		Category category = new Category("Römer",
				"Die Römer waren die Besten der Welt.").save();

		// Add 2 new Books to the Category

		category.addBook("Marc Aurel",
				"Marc Aurel war sehr ein wichtiger Römer.",
				"Buchrücken angerissen", "Gustav Gans", 250, 2);
		category.addBook("Julius Cesar ",
				"Der Monat Juli ist nach Julius Cesar benannt.", "Sehr gut",
				"Donald Duck", 990, 1);

		// Count things
		assertEquals(1, Category.count());
		assertEquals(2, Book.count());

		// Retrieve Category
		category = null;
		category = Category.find("byTitle", "Römer").first();
		assertNotNull(category);

		// Navigate to Books
		assertEquals(2, category.books.size());
		assertEquals("Gustav Gans", category.books.get(0).author);

		// Delete the category
		category.delete();

		// Check that all books have been deleted
		assertEquals(0, Category.count());
		assertEquals(0, Book.count());

	}

	@Test
	public void tryConnectAsUser() {

		// Create a new User and save it
		new User("BookFreak21", "Diamond30", "freak@book.com", "James",
				"Morrison", "Evergreen St. 30", 3400, "Burgdorf").save();

		// Try to connect
		assertNull(User.connect("freak@book.com", "WrongPW"));
		assertNull(User.connect("wrongemail@book.com", "Diamond30"));
		assertNotNull(User.connect("freak@book.com", "Diamond30"));

	}

}
